@@ -1,12 +1,14 @@
-# Copyright (C) 2022 CVAT.ai Corporation
+# Copyright (C) CVAT.ai Corporation
 #
 # SPDX-License-Identifier: MIT
 
 from __future__ import annotations
 
+import json
 import os
+from contextlib import AbstractContextManager
 from pathlib import Path
-from typing import TYPE_CHECKING, Any, Dict, List, Optional, Sequence, Tuple
+from typing import TYPE_CHECKING, Any, Optional
 
 import requests
 import urllib3
@@ -89,6 +91,7 @@ class _MyTusUploader(_TusUploader):
         headers["upload-length"] = str(self.file_size)
         headers["upload-metadata"] = ",".join(self.encode_metadata())
         resp = self._api_client.rest_client.POST(self.client.url, headers=headers)
+        self.real_filename = resp.headers.get("Upload-Filename")
         url = resp.headers.get("location")
         if url is None:
             msg = "Attempt to retrieve create file url with status {}".format(resp.status_code)
@@ -145,9 +148,9 @@ class Uploader:
         url: str,
         filename: Path,
         *,
-        meta: Dict[str, Any],
-        query_params: Dict[str, Any] = None,
-        fields: Optional[Dict[str, Any]] = None,
+        meta: dict[str, Any],
+        query_params: dict[str, Any] = None,
+        fields: Optional[dict[str, Any]] = None,
         pbar: Optional[ProgressReporter] = None,
         logger=None,
     ) -> urllib3.HTTPResponse:
@@ -178,31 +181,23 @@ class Uploader:
         # query params are used only in the extra messages
         assert meta["filename"]
 
+        if pbar is None:
+            pbar = NullProgressReporter()
+
+        file_size = filename.stat().st_size
+
         self._tus_start_upload(url, query_params=query_params)
-        self._upload_file_data_with_tus(
-            url=url, filename=filename, meta=meta, pbar=pbar, logger=logger
-        )
+        with self._uploading_task(pbar, file_size):
+            real_filename = self._upload_file_data_with_tus(
+                url=url, filename=filename, meta=meta, pbar=pbar, logger=logger
+            )
+        query_params["filename"] = real_filename
         return self._tus_finish_upload(url, query_params=query_params, fields=fields)
 
-    def _wait_for_completion(
-        self,
-        url: str,
-        *,
-        success_status: int,
-        status_check_period: Optional[int] = None,
-        query_params: Optional[Dict[str, Any]] = None,
-        post_params: Optional[Dict[str, Any]] = None,
-        method: str = "POST",
-        positive_statuses: Optional[Sequence[int]] = None,
-    ) -> urllib3.HTTPResponse:
-        return self._client.wait_for_completion(
-            url,
-            success_status=success_status,
-            status_check_period=status_check_period,
-            query_params=query_params,
-            post_params=post_params,
-            method=method,
-            positive_statuses=positive_statuses,
+    @staticmethod
+    def _uploading_task(pbar: ProgressReporter, total_size: int) -> AbstractContextManager[None]:
+        return pbar.task(
+            total=total_size, desc="Uploading data", unit_scale=True, unit="B", unit_divisor=1024
         )
 
     @staticmethod
@@ -216,23 +211,18 @@ class Uploader:
 
         return _MyTusUploader(client=client, api_client=api_client, **kwargs)
 
-    def _upload_file_data_with_tus(self, url, filename, *, meta=None, pbar=None, logger=None):
-        file_size = filename.stat().st_size
-        if pbar is None:
-            pbar = NullProgressReporter()
-
-        with open(filename, "rb") as input_file, StreamWithProgress(
-            input_file, pbar, length=file_size
-        ) as input_file_with_progress:
+    def _upload_file_data_with_tus(self, url, filename, *, meta=None, pbar, logger=None) -> str:
+        with open(filename, "rb") as input_file:
             tus_uploader = self._make_tus_uploader(
                 self._client.api_client,
                 url=url.rstrip("/") + "/",
                 metadata=meta,
-                file_stream=input_file_with_progress,
+                file_stream=StreamWithProgress(input_file, pbar),
                 chunk_size=Uploader._CHUNK_SIZE,
                 log_func=logger,
             )
             tus_uploader.upload()
+            return tus_uploader.real_filename
 
     def _tus_start_upload(self, url, *, query_params=None):
         response = self._client.api_client.rest_client.POST(
@@ -267,54 +257,51 @@ class AnnotationUploader(Uploader):
         filename: Path,
         format_name: str,
         *,
-        url_params: Optional[Dict[str, Any]] = None,
+        conv_mask_to_poly: Optional[bool] = None,
+        url_params: Optional[dict[str, Any]] = None,
         pbar: Optional[ProgressReporter] = None,
         status_check_period: Optional[int] = None,
     ):
         url = self._client.api_map.make_endpoint_url(endpoint.path, kwsub=url_params)
         params = {"format": format_name, "filename": filename.name}
-        self.upload_file(
+        response = self.upload_file(
             url, filename, pbar=pbar, query_params=params, meta={"filename": params["filename"]}
         )
 
-        self._wait_for_completion(
-            url,
-            success_status=201,
-            positive_statuses=[202],
-            status_check_period=status_check_period,
-            query_params=params,
-            method="POST",
-        )
+        if conv_mask_to_poly is not None:
+            params["conv_mask_to_poly"] = "true" if conv_mask_to_poly else "false"
+
+        rq_id = json.loads(response.data).get("rq_id")
+        assert rq_id, "The rq_id was not found in the response"
+
+        self._client.wait_for_completion(rq_id, status_check_period=status_check_period)
 
 
 class DatasetUploader(Uploader):
     def upload_file_and_wait(
         self,
         upload_endpoint: Endpoint,
-        retrieve_endpoint: Endpoint,
         filename: Path,
         format_name: str,
         *,
-        url_params: Optional[Dict[str, Any]] = None,
+        url_params: Optional[dict[str, Any]] = None,
+        conv_mask_to_poly: Optional[bool] = None,
         pbar: Optional[ProgressReporter] = None,
         status_check_period: Optional[int] = None,
     ):
         url = self._client.api_map.make_endpoint_url(upload_endpoint.path, kwsub=url_params)
         params = {"format": format_name, "filename": filename.name}
-        self.upload_file(
+
+        if conv_mask_to_poly is not None:
+            params["conv_mask_to_poly"] = "true" if conv_mask_to_poly else "false"
+
+        response = self.upload_file(
             url, filename, pbar=pbar, query_params=params, meta={"filename": params["filename"]}
         )
+        rq_id = json.loads(response.data).get("rq_id")
+        assert rq_id, "The rq_id was not found in the response"
 
-        url = self._client.api_map.make_endpoint_url(retrieve_endpoint.path, kwsub=url_params)
-        params = {"action": "import_status"}
-        self._wait_for_completion(
-            url,
-            success_status=201,
-            positive_statuses=[202],
-            status_check_period=status_check_period,
-            query_params=params,
-            method="GET",
-        )
+        self._client.wait_for_completion(rq_id, status_check_period=status_check_period)
 
 
 class DataUploader(Uploader):
@@ -325,56 +312,59 @@ class DataUploader(Uploader):
     def upload_files(
         self,
         url: str,
-        resources: List[Path],
+        resources: list[Path],
         *,
         pbar: Optional[ProgressReporter] = None,
         **kwargs,
     ):
         bulk_file_groups, separate_files, total_size = self._split_files_by_requests(resources)
 
-        if pbar is not None:
-            pbar.start(total_size, desc="Uploading data")
+        if pbar is None:
+            pbar = NullProgressReporter()
 
-        self._tus_start_upload(url)
+        if str(kwargs.get("sorting_method")).lower() == "predefined":
+            # Request file ordering, because we reorder files to send more efficiently
+            kwargs.setdefault("upload_file_order", [p.name for p in resources])
 
-        for group, group_size in bulk_file_groups:
-            files = {}
-            for i, filename in enumerate(group):
-                files[f"client_files[{i}]"] = (
-                    os.fspath(filename),
-                    filename.read_bytes(),
+        with self._uploading_task(pbar, total_size):
+            self._tus_start_upload(url)
+
+            for group, group_size in bulk_file_groups:
+                files = {}
+                for i, filename in enumerate(group):
+                    files[f"client_files[{i}]"] = (
+                        os.fspath(filename),
+                        filename.read_bytes(),
+                    )
+                response = self._client.api_client.rest_client.POST(
+                    url,
+                    post_params={"image_quality": kwargs["image_quality"], **files},
+                    headers={
+                        "Content-Type": "multipart/form-data",
+                        "Upload-Multiple": "",
+                        **self._client.api_client.get_common_headers(),
+                    },
                 )
-            response = self._client.api_client.rest_client.POST(
-                url,
-                post_params={"image_quality": kwargs["image_quality"], **files},
-                headers={
-                    "Content-Type": "multipart/form-data",
-                    "Upload-Multiple": "",
-                    **self._client.api_client.get_common_headers(),
-                },
-            )
-            expect_status(200, response)
+                expect_status(200, response)
 
-            if pbar is not None:
                 pbar.advance(group_size)
 
-        for filename in separate_files:
-            # TODO: check if basename produces invalid paths here, can lead to overwriting
-            self._upload_file_data_with_tus(
-                url,
-                filename,
-                meta={"filename": filename.name},
-                pbar=pbar,
-                logger=self._client.logger.debug,
-            )
+            for filename in separate_files:
+                self._upload_file_data_with_tus(
+                    url,
+                    filename,
+                    meta={"filename": filename.name},
+                    pbar=pbar,
+                    logger=self._client.logger.debug,
+                )
 
-        self._tus_finish_upload(url, fields=kwargs)
+        return self._tus_finish_upload(url, fields=kwargs)
 
     def _split_files_by_requests(
-        self, filenames: List[Path]
-    ) -> Tuple[List[Tuple[List[Path], int]], List[Path], int]:
-        bulk_files: Dict[str, int] = {}
-        separate_files: Dict[str, int] = {}
+        self, filenames: list[Path]
+    ) -> tuple[list[tuple[list[Path], int]], list[Path], int]:
+        bulk_files: dict[str, int] = {}
+        separate_files: dict[str, int] = {}
         max_request_size = self.max_request_size
 
         # sort by size
@@ -389,9 +379,9 @@ class DataUploader(Uploader):
         total_size = sum(bulk_files.values()) + sum(separate_files.values())
 
         # group small files by requests
-        bulk_file_groups: List[Tuple[List[str], int]] = []
+        bulk_file_groups: list[tuple[list[str], int]] = []
         current_group_size: int = 0
-        current_group: List[str] = []
+        current_group: list[str] = []
         for filename, file_size in bulk_files.items():
             if max_request_size < current_group_size + file_size:
                 bulk_file_groups.append((current_group, current_group_size))
